function Get-VMKvp {
    <#
    
    .SYNOPSIS
    Get the Key Value Exchange properties for the passed in virtual machine.
    
    .DESCRIPTION
    Get the Key Value Exchange properties for the passed in virtual machine from this server.
    The supported Operating Systems are Window Server 2012, Windows Server 2012R2, Windows Server 2016.
    
    .ROLE
    Readers
    
    .PARAMETER vmId
          The Id of the requested virtual machine.
    
    #>
    
    param (
        [Parameter(ValueFromPipelineByPropertyName)]

        $ID
    )  
    process {
    Set-StrictMode -Version 5.0;
    import-module CimCmdlets;
    Import-Module Hyper-V;
    $vmId = $ID
    $vm=get-vm -id $vmId;
    
    # define a new object that will hold the properties that will be returned to the caller.
    $vmProperties = New-Object psobject
        
    Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "id" -Value $vmId
    Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "IsClustered" -Value $vm.isClustered;
    
    $checkpoints=@($vm | Get-VMSnapshot | Sort-Object creationtime);
    
    if ($checkpoints.Count -gt 0) {
        $lastCheckpointTime=$checkpoints[$checkpoints.Count-1].CreationTime;
        Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "LastSuccessfullCheckpoint" -Value $lastCheckpointTime;
    } else {
        Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "LastSuccessfullCheckpoint" -Value $null;
    }
    
    # get the CIM VM instance
    $cimVm = Get-CimInstance -namespace root\virtualization\v2 -ClassName Msvm_ComputerSystem -filter "Name = '$($vmId)'";
    
    # get the associated KVP exchange component
    $Kvp = Get-CimAssociatedInstance -InputObject $cimVm -Association Msvm_SystemDevice -ResultClassName Msvm_KvpExchangeComponent;
    
    # extract the values from the KVP exchange and add as properties to the object that was created for this task.
    try {
        Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "OSName" -Value (([xml]($Kvp.GuestIntrinsicExchangeItems | where-object { $_ -match "OSName"})).instance.property | where-object {$_.Name -eq "Data"}).Value;
    }
    catch [System.Management.Automation.PropertyNotFoundException] {
        Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "OSName" -Value "";
    }
    
    try {
        Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "OSVersion" -Value (([xml]($Kvp.GuestIntrinsicExchangeItems | where-object { $_ -match "OSVersion"})).instance.property | where-object {$_.Name -eq "Data"}).Value;
    }
    catch [System.Management.Automation.PropertyNotFoundException] {
        Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "OSVersion" -Value "";
    }
    
    try {
        Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "FullyQualifiedDomainName" -Value (([xml]($Kvp.GuestIntrinsicExchangeItems | where-object { $_ -match "FullyQualifiedDomainName"})).instance.property | where-object {$_.Name -eq "Data"}).Value;
    }
    catch [System.Management.Automation.PropertyNotFoundException] {
        Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "FullyQualifiedDomainName" -Value "";
    }
    
    try {
        Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "IntegrationServicesVersion" -Value (([xml]($Kvp.GuestIntrinsicExchangeItems | where-object { $_ -match "IntegrationServicesVersion"})).instance.property | where-object {$_.Name -eq "Data"}).Value;
    }
    catch [System.Management.Automation.PropertyNotFoundException] {
        Add-Member -InputObject $vmProperties -MemberType NoteProperty -Name "IntegrationServicesVersion" -Value "";
    }
    
      $vmProperties}
    }
    